package GD::SecurityImage::GD;
use strict;
use vars qw[$VERSION $methTTF];

use constant LOW_LEFT_X  => 0; # Lower left  corner x
use constant LOW_LEFT_Y  => 1; # Lower left  corner y
use constant LOW_RIGHT_X => 2; # Lower right corner x
use constant LOW_RIGHT_Y => 3; # Lower right corner y
use constant UP_RIGHT_X  => 4; # Upper right corner x
use constant UP_RIGHT_Y  => 5; # Upper right corner y
use constant UP_LEFT_X   => 6; # Upper left  corner x
use constant UP_LEFT_Y   => 7; # Upper left  corner y

use GD;

$VERSION = "1.4";
$methTTF = $GD::VERSION >= 1.31 ? 'stringFT' : 'stringTTF'; # define the tff drawing method.

sub init {
   # Create the image object
   my $self = shift;
      $self->{image} = GD::Image->new($self->{width}, $self->{height});
      $self->{image}->colorAllocate(@{ $self->{bgcolor} }); # set background color
      $self->setThickness($self->{thickness}) if $self->{thickness};
}

sub out {
   # return $image_data, $image_mime_type, $random_number
   my $self = shift;
   my %opt  = scalar @_ % 2 ? () : (@_);
   my $type;
   if($opt{force} and $self->{image}->can($opt{force})){
      $type = $opt{force};
   } else {
      $type = $self->{image}->can('gif') ? 'gif' : 'jpeg'; # check for older GDs and newer GDs as the new versions include gif() again!
   }
   return $self->{image}->$type(), $type, $self->{_RANDOM_NUMBER_};
}

sub gdbox_empty {shift->{GDBOX_EMPTY}}

sub gdfx {
   # Sets the font for simple GD usage. 
   # Unfortunately, Image::Magick does not have a similar interface.
   my $self = shift;
   my $font = shift || return;
      $font = lc $font;
   # GD' s standard fonts
   my %f = map { lc $_ => $_ } qw[ Small Large MediumBold Tiny Giant ];
   if (exists $f{$font}) {
      $font = $f{$font};
      return GD::Font->$font();
   }
   return;
}

sub insert_text {
   # Draw text using GD
   my $self   = shift;
   my $method = shift;
   my $key    = $self->{_RANDOM_NUMBER_}; # random string
   if ($method eq 'ttf') {
      require Math::Trig;
      # don' t draw. we just need info...
      my $info = sub {
                      my $txt = shift;
                      my $ang = shift || 0;
                         $ang = Math::Trig::deg2rad($ang) if $ang;
                      my @box = GD::Image->$methTTF($self->{_COLOR_}{text},$self->{font},$self->{ptsize},$ang,0,0,$txt);
                      unless (@box) { # use fake values instead of die-ing
                         $self->{GDBOX_EMPTY} = 1; # set this for error checking.
                         $#box    = 7;
                         # lets initialize to silence the warnings
                         $box[$_] = 1 for 0..7;
                      }
                      return @box;
      };
      if ($self->{scramble}) {
         my @char;
         my $anglex;
         my $total = 0;
         my $space = [$self->ttf_info(0, 'A'),0,'  '];
         foreach (split //, $key) { # get char parameters
            $anglex = $self->random_angle;
            $total += $space->[0];
            push @char, [$self->ttf_info($anglex, $_), $anglex, $_], $space, $space, $space;
         }
         $total *= 2;
         my @config = ($self->{_COLOR_}{text}, $self->{font}, $self->{ptsize});
         my($x,$y);
         foreach my $box (reverse @char) {
            $x = $self->{width}  / 2 + ($box->[0] - $total);
            $y = $self->{height} / 2 +  $box->[1];
            $self->{image}->$methTTF(@config, Math::Trig::deg2rad($box->[2]), $x, $y, $box->[3]);
            $total -= $space->[0];
         }
      } else {
         my @box = $info->($key);
         my $x   = ($self->{width}  - ($box[LOW_RIGHT_X] - $box[LOW_LEFT_X])) / 2;
         my $y   = ($self->{height} - ($box[UP_LEFT_Y]   - $box[LOW_LEFT_Y])) / 2;
         $self->{image}->$methTTF($self->{_COLOR_}{text}, $self->{font}, $self->{ptsize}, 0, $x, $y, $key);
      }
   } else {
      if ($self->{scramble}) {
         # without ttf, we can only have 0 and 90 degrees.
         my @char;
         my @styles = qw(string stringUp);
         my $style  = $styles[int rand @styles];
         foreach (split //, $key) { # get char parameters
            push @char, [$_, $style], [' ','string'];
            $style = $style eq 'string' ? 'stringUp' : 'string';
         }
         my $sw = $self->{gd_font}->width;
         my $sh = $self->{gd_font}->height;
         my($x, $y, $m);
         my $total = $sw * @char;
         foreach my $c (@char) {
            $m = $c->[1];
            $x = ($self->{width}  - $total) / 2;
            $y = $self->{height}/2 + ($m eq 'string' ? -$sh : $sh/2) / 2;
            $total -= $sw * 2;
            $self->{image}->$m($self->{gd_font}, $x, $y, $c->[0], $self->{_COLOR_}{text});
         }
      } else {
         my $sw = $self->{gd_font}->width * length($key);
         my $sh = $self->{gd_font}->height;
         my $x  = ($self->{width}  - $sw) / 2;
         my $y  = ($self->{height} - $sh) / 2;
         $self->{image}->string($self->{gd_font}, $x, $y, $key, $self->{_COLOR_}{text});
      }
   }
}

sub ttf_info {
   my $self  = shift;
   my $angle = shift || 0;
   my $text  = shift;
   my $x     = 0;
   my $y     = 0;
   my @box = GD::Image->$methTTF($self->{_COLOR_}{text},$self->{font}, $self->{ptsize},Math::Trig::deg2rad($angle),0,0,$text);
   my $bx  = $box[LOW_LEFT_X] - $box[LOW_RIGHT_X];
   my $by  = $box[LOW_LEFT_Y] - $box[LOW_RIGHT_Y];
      $by  = $box[ UP_LEFT_Y] - $box[LOW_LEFT_Y ] if $angle == 0  or $angle == 180 or $angle == 360;
      $bx  = $box[ UP_LEFT_X] - $box[LOW_LEFT_X ] if $angle == 90 or $angle == 270;
      if ($angle ==   0                 ) { $x += $bx/2; $y -= $by/2;   }
   elsif ($angle >    0 and $angle < 90 ) {              $y += $by;     }
   elsif ($angle ==  90                 ) { $x -= $bx/2; $y += $by/2;   }
   elsif ($angle >   90 and $angle < 180) { $x += $bx;                  }
   elsif ($angle == 180                 ) { $x += $bx/2; $y -= $by/2;   }
   elsif ($angle >  180 and $angle < 270) {              $y += $by;     }
   elsif ($angle == 270                 ) { $x -= $bx/2; $y += $by/2;   }
   elsif ($angle >  270 and $angle < 360) { $x += $bx;                  }
   elsif ($angle == 360                 ) { $x += $bx/2; $y -= $by/2;   }
   return $x, $y;
}

sub setPixel        {shift->{image}->setPixel(@_)       }
sub line            {shift->{image}->line(@_)           }
sub rectangle       {shift->{image}->rectangle(@_)      }
sub filledRectangle {shift->{image}->filledRectangle(@_)}
sub ellipse         {shift->{image}->ellipse(@_)        }
sub arc             {shift->{image}->arc(@_)            }
sub setThickness    {shift->{image}->setThickness(@_)   }

1;

__END__

=head1 NAME

GD::SecurityImage::GD - GD backend for GD::SecurityImage.

=head1 SYNOPSIS

See L<GD::SecurityImage>.

=head1 DESCRIPTION

Used internally by L<GD::SecurityImage>. Nothing public here.

=head1 SEE ALSO

L<GD::SecurityImage>.

=head1 AUTHOR

Burak Grsoy, E<lt>burakE<64>cpan.orgE<gt>

=head1 COPYRIGHT

Copyright 2004 Burak Grsoy. All rights reserved.

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
